# Copyright (C) 2022 RozK
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from array import array

from engine import (
    INSTANCE_FLAG_SPAWNED, BATCH_MAX_SIZE, create_batch, draw_batch, destroy_batch)

class Batch:
    __slots__ = '_batch', 'max_size', 'format', 'flags', 'meshes', 'params'

    def __init__(self, vertices, max_size, params_format):
        assert max_size <= BATCH_MAX_SIZE
        self._batch = create_batch(vertices, max_size, params_format)
        self.max_size = max_size
        self.format = params_format
        self.flags = array('B')
        self.meshes = array('I')
        self.params = array('f')

    def __del__(self):
        destroy_batch(self._batch)

    def append(self, flags, mesh, params):
        assert len(params) == len(self.format)
        index = len(self.flags)
        assert index < self.max_size
        self.flags.append(flags | INSTANCE_FLAG_SPAWNED)
        self.meshes.append(mesh)
        def append_param(param):
            start = len(self.params)
            self.params.extend(param)
            return slice(start, len(self.params))
        return tuple(map(append_param, params))

    def set_param(self, param, id, value):
        self.params[id[param]] = value

    def draw(self):
        draw_batch(self._batch, self.flags, self.meshes, self.params)
